unit JumboDialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, PalForm, comctrls;

type
  TPalette = array [0..$FFFF] of integer;
  PPalette = ^TPalette;

  TJumboDialog = class(TCommonDialog)
  private
    { Private declarations }
  	PaletteForm: TPaletteForm;

    FUserOK : boolean;

    FPalPath : string;
    FPalSize : integer;
	FMaxRecentCount : integer;
    FbShowToolz : boolean;
    FbLargeIcons : boolean;    
    FDefUsrTool : TWinControl;
    FCaption : string;
    procedure SetPalPath( s : string );
    procedure SetPalSize( n : integer );
    procedure SetMaxRecentCount( n : integer );
    function GetRecentCount : integer;
    function GetRecent( index : integer ) : pointer;
    function GetRecentBitmap : TBitmap;
    function GetResultPtr : pointer;
    function GetResultPal : PPalette;
    function GetPalList : TListView;
    function GetHandle : Hwnd;
    function GetUserToolzRebar : TCoolBar;
    function GetEditMask : integer;
    procedure SetEditMask( new_mask : integer );
  protected
    { Protected declarations }
  public
    { Public declarations }
    property UserOK : boolean read FUserOK;
    property RecentCount : integer read GetRecentCount;
    property Recent[index : integer ] : pointer read GetRecent;
	property RecentBitmap : TBitmap read GetRecentBitmap;
    property ResultPtr : pointer read GetResultPtr;
    property ResultPal : PPalette read GetResultPal;

    property Handle : Hwnd read GetHandle;

    {executes the dialog. Returns true if user confirmed his choice.}
	function Execute: Boolean;

    function StoreResult( dest : pointer ) : boolean;
    function ScaleResult( dest : pointer; destwidth : integer; flipRGB : boolean ) : boolean;
    function DupResult( original_dest : pointer; destwidth : integer; flipRGB : boolean ) : pointer;

    procedure Restart;

    {API}
    property PalList : TListView read GetPalList;
    property EditMask : integer read GetEditMask write SetEditMask;

    property JumboUserToolz : TCoolBar read GetUserToolzRebar;

    procedure ActivateToolOptions( your_options : TWinControl; title : string  );

    {this routines works with the items of __PalList}
    procedure SubstitutePalBitmap( index : integer; pal : pointer );
    procedure UpdatePalBitmap( index : integer );

    function GetActivePalette : PPalette;
    procedure ShowActivePalette( pal : ppalette; caption : string  );
    procedure RepaintActivePalette( everywhere : boolean );
    function CreatePalBitmap( bmp : TBitmap; width,height : integer; frame_width : integer; pal : ppalette ) : TBitmap;


{    procedure SetEditMask( newmask : integer );
    procedure SubstitutePalBitmap( index : integer; pal : pointer );
}
    {Palette Folder = PF. Each PF, when saved, is just a bmp in _PalettePath_ directory}
    {Folders are numerated 0..GetFoldersCount-1}

    function GetFolderID( FolderName : string ) : integer;
    function GetFolderName( folder_id : integer ) : string;
    function GetActiveFolder : integer; {-1 - no folder; 0..Max - folder id}
    function GetFoldersCount : integer;

    function CreateNewFolder( DesiredFolderName : string ) : integer;
    procedure DeleteFolder( folder_id : integer );
    function ActivateFolder( folder_id : integer ) : boolean;

	procedure AddToFolder( folder_id : integer; p : Pointer );
    procedure ClearFolder( folder_id : integer );

    function GetFolderSize( folder_id : integer ) : integer;
    function GetFolderPalette( folder_id : integer; index : integer ) : PPalette;

(*
    {Creates new PF. If PF with this name exists, returns False}
	function CreateNewFolder( DesiredFolderName : string ) : boolean;
    {Activates PF, e.g. fills PalList, etc. False if no such folder.}
    function ActivateFolder( FolderName : string ) : boolean;
    {Adds palette _p to PF. _p, of course, is a ptr to an array of _palSize dwords}
	procedure AddToFolder( FolderName : string; p : Pointer );
    {Clears PF}
    procedure ClearFolder( FolderName : string );
    {Deletes PF. In actual fact, sets "cut" flag to PF - it will be deleted on next _saveall procedure}
    procedure DeleteFolder( FolderName : string );
    {Returns PF's size. -1 if no such a folder.}
    function GetFolderSize( FolderName : string ) : integer;
    {Returns ptr to palette in PF, indexed by _index (starting from 0). Nil on error.}
    function GetFolderPalette( FolderName : string; index : integer ) : PPalette;

    {}
    function GetFoldersCount : integer;
*)

    procedure Loaded; override;
  published
    constructor Create(AOwner: TComponent); override;

    property Caption : string read FCaption write FCaption;
  	property PalettePath : string read FPalPath write SetPalPath;
    property PaletteSize : integer read FPalSize write SetPalSize default 1024;
    property MaxRecentCount : integer read FMaxRecentCount write SetMaxRecentCount default 32;
    property bShowToolz : boolean read FbShowToolz write FbShowToolz default True;
    property bLargeFoldersIcons : boolean read FbLargeIcons write FbLargeIcons default True;

  	property DefaultUserToolz : TWinControl read FDefUsrTool write FDefUsrTool;


    { Published declarations }
  end;

procedure Register;
function _rgbflip( c : integer ) : integer;

implementation

procedure Register;
begin
  RegisterComponents('Dialogs', [TJumboDialog]);
end;

function _rgbflip( c : integer ) : integer;
begin
	Result := ((c and $FF) shl 16)
    		or( c and $FF00) or ((c shr 16) and $FF);
end;

constructor TJumboDialog.Create(AOwner: TComponent);
begin
	inherited Create( AOwner );
    FUserOK := false;
    FPalPath := 'Palettes\';
    FPalSize := 1024;
    FMaxRecentCount := 32;
    FbShowToolz := True;
    FbLargeIcons := True;
    FCaption := 'Select palette...';    
end;

function TJumboDialog.Execute: Boolean;
begin
	PaletteForm.Caption := FCaption;
	FUserOK := PaletteForm.ShowModal = mrOK;
	Result := FUserOK;
end;

function TJumboDialog.GetPalList : TListView;
begin
	result := PaletteForm.PalList;
end;

function TJumboDialog.GetHandle : Hwnd;
begin
	if PaletteForm = nil then Result := 0 else
    	Result := PaletteForm.Handle;
end;

procedure TJumboDialog.SetPalPath( s : string );
begin
    if s[ length(s) ] <> '\' then
    	FPalPath := s+'\' else FPalPath := s;
end;

procedure TJumboDialog.SetPalSize( n : integer );
begin
	if n < 1 then n := 1
    else if n > $FFFF then n := $FFFF;
    FPalSize := n;
end;

function TJumboDialog.GetRecentCount : integer;
begin
	if PaletteForm.recentBitmap = nil then
    	Result := 0
    else Result := PaletteForm.recentBitmap.Height;
end;

function TJumboDialog.GetRecent( index : integer ) : pointer;
begin
	if PaletteForm.recentBitmap = nil
    then Result := nil else
    if (index < 0) or (index >= PaletteForm.recentBitmap.Height)
    then Result := nil else
    Result := PaletteForm.recentBitmap.ScanLine[ index ];
end;

function TJumboDialog.GetResultPtr : pointer;
begin
	Result := Recent[0];
end;

function TJumboDialog.GetResultPal : PPalette;
begin
	Result := Recent[0];
end;


function TJumboDialog.GetRecentBitmap : TBitmap;
begin
    Result := PaletteForm.recentBitmap;
end;

procedure TJumboDialog.SetMaxRecentCount( n : integer );
begin
	if n < 1 then n := 1
    else if n > $FFFF then n := $FFFF;
    FmaxRecentCount := n;
end;

function TJumboDialog.StoreResult( dest : pointer ) : boolean;
begin
	Result := true;
	if RecentCount <= 0 then Result := False
    else Move( ResultPtr^,dest^,fpalSize*4 );
end;

function TJumboDialog.ScaleResult( dest : pointer; destwidth : integer; flipRGB : boolean ) : boolean;
var i : integer;
begin
	Result := False;
	if RecentCount <= 0 then exit;
    if destwidth = fpalSize then StoreResult( dest )
     	else PaletteForm.ResizeBits( ResultPtr,dest,fpalSize,destwidth );
	if flipRGB then
    	for i := 0 to destwidth-1 do
        	PPalette(dest)[i] := _rgbflip( PPalette(dest)[i] );
	Result := True;
end;

function TJumboDialog.DupResult( original_dest : pointer; destwidth : integer; flipRGB : boolean ) : pointer;
begin
	ReallocMem( original_dest, destwidth*4 );
    ScaleResult( original_dest,destwidth,flipRGB );
    Result := original_dest;
end;

function TJumboDialog.GetUserToolzRebar : TCoolBar;
begin
	Result := PaletteForm.ToolzRebar;
end;

procedure TJumboDialog.Restart;
begin
	if PaletteForm = nil then begin
    	PaletteForm := TPaletteForm.Create( Self );
	   	if FDefUsrTool <> nil then
        	 FDefUsrTool.Parent := JumboUserToolz;
    end
    else PaletteForm.Shutdown.Execute;

	PaletteForm.palPath := FPalPath;
   	PaletteForm.palSize := FPalSize;
    PaletteForm.maxRecentCount := FmaxRecentCount;
    PaletteForm.bShowToolz := FbShowToolz;
    PaletteForm.bLargeIcons := FbLargeIcons;
    PaletteForm.Init.Execute;
end;


procedure TJumboDialog.Loaded;
begin
	inherited Loaded;

    if csDesigning in ComponentState then begin
    end else Restart;
end;


function TJumboDialog.GetFolderID( FolderName : string ) : integer;
begin
	Result := PaletteForm.GetFolderID( FolderName );
end;

function TJumboDialog.GetFolderName( folder_id : integer ) : string;
begin
	Result := PaletteForm.GetFolderName( folder_id )
end;

function TJumboDialog.GetActiveFolder : integer; {-1 - no folder; 0..Max - folder id}
begin
	Result := PaletteForm.GetActiveFolder
end;

function TJumboDialog.GetFoldersCount : integer;
begin
	Result := PaletteForm.GetFoldersCount
end;

function TJumboDialog.CreateNewFolder( DesiredFolderName : string ) : integer;
begin
	Result := PaletteForm.CreateNewFolder( DesiredFolderName )
end;

procedure TJumboDialog.DeleteFolder( folder_id : integer );
begin
	PaletteForm.DeleteFolder( folder_id );
end;

function TJumboDialog.ActivateFolder( folder_id : integer ) : boolean;
begin
	Result := PaletteForm.ActivateFolder( folder_id );
end;

procedure TJumboDialog.AddToFolder( folder_id : integer; p : Pointer );
begin
	PaletteForm.AddToFolder( folder_id ,p );
end;

procedure TJumboDialog.ClearFolder( folder_id : integer );
begin
	PaletteForm.ClearFolder( folder_id );
end;

function TJumboDialog.GetFolderSize( folder_id : integer ) : integer;
begin
	Result := PaletteForm.GetFolderSize( folder_id )
end;

function TJumboDialog.GetFolderPalette( folder_id : integer; index : integer ) : PPalette;
var p : PPalette;
begin
	p := Pointer( PaletteForm.GetFolderPalette( folder_id,index ) );
	Result := p;
end;

function TJumboDialog.GetEditMask : integer;
begin
	Result := PaletteForm.nEditMask;
end;

procedure TJumboDialog.SetEditMask( new_mask : integer );
begin
	PaletteForm.SetEditMask( new_mask );
end;

procedure TJumboDialog.SubstitutePalBitmap( index : integer; pal : pointer );
begin
	PaletteForm.SubstitutePalBitmap( index,pal);
end;

procedure TJumboDialog.UpdatePalBitmap( index : integer );
begin
	PaletteForm.UpdatePalBitmap( index );
end;

function TJumboDialog.GetActivePalette : PPalette;
begin
	Result := Pointer(PaletteForm.GetActivePalette);
end;

procedure TJumboDialog.ShowActivePalette( pal : ppalette; caption : string  );
begin
	PaletteForm.ShowActivePalette( pal,caption );
end;

procedure TJumboDialog.RepaintActivePalette( everywhere : boolean );
begin
	PaletteForm.RepaintActivePalette( everywhere );
end;

function TJumboDialog.CreatePalBitmap( bmp : TBitmap; width,height : integer; frame_width : integer; pal : ppalette ) : TBitmap;
begin
	Result := PaletteForm.CreatePalBitmap( bmp,width,height,frame_width,Pointer(pal) );
end;

procedure TJumboDialog.ActivateToolOptions( your_options : TWinControl; title : string  );
begin
	PaletteForm.ActivateToolOptions( your_options,title );
end;

end.
